------------------------------------
-- Shot Glass Raid Frames
------------------------------------
local addonname = "ShotGlass"
local artpath = "Interface\\Addons\\ShotGlass\\Art\\"
local rangePollFrequency = .6
local sgShown = true
local sgConfig = {
	ShowDrag = true,
	ShowSolo = true,
	ShowParty = true,
	ShowRaid = true,
	ShowPlayer = true,
	Defaultscale = 1,
	Point = "LEFT",
}
--sgConfiguration = nil
-- Clique Support
ClickCastFrames = ClickCastFrames or {}											

-- Function List
local UpdateButton, OnShowButton, OnChangeAttribute, OnEvent, SetHandlers		
local CreateGroupHeader, CreateUnitFrameArtwork
local CallArtworkFunction, SecureInitFunction

------------------------------------
-- Aura Tracking List
------------------------------------
local TrackedAuras = {}

------------------------------------
-- Helper Functions
------------------------------------

-- Abbreviates the Unit Name
local function TruncateName(text) if text then return text:gsub("(%u%l%l%l%l)(.*)", "%1") end end

-- Resets all Indicators
local function ResetIndicators(self) 
	local name, indicator
	for name, indicator in pairs(self.Graphics.Indicators) do 
		if type(indicator)=='table' then indicator:Hide() end
	end
end

-- Polls for Range Check
local function PollOnUpdate(self)
	local unitid = self.UnitID
	if not unitid then return end
	local now = GetTime()
	if now > self.rangeUpdateTime then self.rangeUpdateTime = now + rangePollFrequency else return end
	
	-- Range Check
	if UnitInRange(unitid) then self.Graphics:SetScale(sgConfig.Defaultscale)
	else self.Graphics:SetScale(.6) end
end

-- Hides Party Frames
for i=1,4 do
	local frame = _G["PartyMemberFrame"..i]
	frame:UnregisterAllEvents()
	frame.Show = function() end
	frame:Hide()
end

-- Applies Color and Stack text to an Indicator
local function ApplyIndicatorStyle(indicator, color, stacks, timeleft, index)
	-- if index < 4 then
		-- if timeleft > 1000 then
			-- indicator.Image:SetTexture(artpath.."DebuffDot")
		-- else
			-- indicator.Image:SetTexture(artpath.."DebuffBigDot")
		-- end
	-- else
		-- if timeleft > 1000 then
			-- indicator.Image:SetTexture(artpath.."DebuffArrow")
		-- else
			-- indicator.Image:SetTexture(artpath.."DebuffBigArrow")
		-- end
	-- end --if
	-- Color
	indicator.Image:SetVertexColor(color.r,color.g,color.b,color.a)
	-- Stack Text
	if stacks and (stacks > 0) then indicator.Text:SetText(stacks) 
	else indicator.Text:SetText("") end
	-- Show
	indicator:Show()
end

------------------------------------
-- Events
------------------------------------
local UnitEventHandlers = {}

function UnitEventHandlers.UNIT_AURA(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	
	local _, auraindex, name, debuffType, caster , stacks, indicator, aura, expires
	local priority = {0,0,0,0,0,0,}
	
	if unit == unitid then 
		ResetIndicators(self)
		-- Check Specific Auras
		for auraindex = 1, 40 do
			name, _, _, stacks, _, expires, _, caster = UnitAura(unitid,auraindex,"PLAYER")
			if not name then break end
			aura = TrackedAuras[name] 
			if aura then 
				if not aura.priority or aura.priority > priority[aura.position] then
					indicator = self.Graphics.Indicators[aura.position]
					ApplyIndicatorStyle(indicator, aura.color, stacks, expires-GetTime(), aura.position)
					priority[aura.position] = aura.priority
				end
			end
		end
		-- Check Debuff Types
		for auraindex = 1, 40 do
			name, _, _, stacks, debuffType, _, expires = UnitDebuff(unitid,auraindex)
			if not name then break end
			aura = TrackedAuras[name] or TrackedAuras[debuffType]
			if aura then 
				if not aura.priority or aura.priority > priority[aura.position] then
					indicator = self.Graphics.Indicators[aura.position]
					ApplyIndicatorStyle(indicator, aura.color, stacks, expires-GetTime(), aura.position)
					priority[aura.position] = aura.priority
				end
			end
		end
		
	end
end

function UnitEventHandlers.UNIT_DISPLAYPOWER(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		local _, powertype = UnitPowerType(unitid or "")
		if powertype == "MANA" then
			self.Graphics.Power:Show()
			self.Graphics.Power:SetMinMaxValues(0,UnitPowerMax(unitid))
			self.Graphics.Power:SetValue(UnitPower(unitid))
		else
			self.Graphics.Power:Hide()
		end
	end
end

function UnitEventHandlers.UNIT_POWER(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		self.Graphics.Power:SetValue(UnitPower(unitid))
	end
end

function UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then
		if UnitIsDead(unitid) or UnitIsGhost(unitid) or (not UnitIsConnected(unitid))then 
			self.Graphics.Health:SetAlpha(.7)
		else 
			self.Graphics.Health:SetAlpha(1)
		end
	end
end
UnitEventHandlers.PARTY_MEMBER_ENABLE = UnitEventHandlers.PARTY_MEMBER_DISABLE
UnitEventHandlers.PLAYER_DEAD = UnitEventHandlers.PARTY_MEMBER_DISABLE

function UnitEventHandlers.UNIT_HEALTH(self, eventunit)
	local unitid = self.UnitID
	if not unitid then return end
	local petunit, visibleunit
	if unitid == "player" then petunit = "pet" else petunit = unitid.."pet" end

	if eventunit == unitid or eventunit == petunit then 
		if UnitHasVehicleUI(unitid) then visibleunit = petunit
		else visibleunit = unitid end
		if UnitExists(visibleunit) then
			-- Sets Health Bar
			self.Graphics.Health:SetMinMaxValues(0,UnitHealthMax(visibleunit))
			self.Graphics.Health:SetValue(UnitHealth(visibleunit))	
		end
	end
end
UnitEventHandlers.UNIT_MAXHEALTH = UnitEventHandlers.UNIT_HEALTH

function UnitEventHandlers.PLAYER_TARGET_CHANGED(self) 
	local unitid = self.UnitID
	if not unitid then return end
	local visibleunit
	
	if UnitHasVehicleUI(unitid) then
		if unitid == "player" then visibleunit = "pet" else visibleunit = unitid.."pet" end
	else visibleunit = unitid end
	
	-- Sets Target Highlight
	if UnitIsUnit("target", visibleunit) then self.Graphics.Border:Show()
	else self.Graphics.Border:Hide() end		
end

function UnitEventHandlers.UNIT_THREAT_SITUATION_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	local unitaggro = UnitThreatSituation(unitid)
	if unitaggro and unitaggro > 1 then 
		self.Graphics.Aggro:Show() 
	else 
		self.Graphics.Aggro:Hide() 
	end
end

function UnitEventHandlers.UNIT_PORTRAIT_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
	UnitEventHandlers.UNIT_HEALTH(self, unitid)
end
UnitEventHandlers.PLAYER_UNGHOST = UnitEventHandlers.UNIT_PORTRAIT_UPDATE
UnitEventHandlers.PLAYER_ALIVE = UnitEventHandlers.UNIT_PORTRAIT_UPDATE

function UnitEventHandlers.UNIT_FLAGS(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)
	
end

function UnitEventHandlers.UNIT_ENTERED_VEHICLE(self, unit)
	OnChangeAttribute(self, "unit")
	if self.UnitID then UpdateButton(self, self.UnitID) end
end
UnitEventHandlers.UNIT_EXITED_VEHICLE = UnitEventHandlers.UNIT_ENTERED_VEHICLE
UnitEventHandlers.UNIT_PET = UnitEventHandlers.UNIT_ENTERED_VEHICLE
function UnitEventHandlers.UNIT_HEAL_PREDICTION(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	
	if unit == unitid then
		local incomingHeal = UnitGetIncomingHeals(unitid) or 0
		if incomingHeal > 0 then incomingHeal = UnitHealth(unitid)+incomingHeal end
		self.Graphics.IncomingHeal:SetValue(incomingHeal)	
	end
end

------------------------------------
-- Update Functions
------------------------------------
function UpdateButton(self, unitid)
		local class, color, Graphics
		self.Nextupdate = 0
		Graphics = self.Graphics
		-- Colors
		class = select(2, UnitClass(unitid)) or "WARRIOR"
		color = RAID_CLASS_COLORS[class]
		-- Name
		Graphics.Text:SetText(TruncateName(UnitName(unitid)))
		-- Health
		local maxhealth = UnitHealthMax(unitid)
		Graphics.Health:SetStatusBarColor(0.05, 0.05, 0.05, 1)
		Graphics.Health:SetMinMaxValues(0,maxhealth)
		-- Incoming Heals
		Graphics.IncomingHeal:SetStatusBarColor(0.05, 0.05, 0.05, .6)
		Graphics.IncomingHeal:SetMinMaxValues(0,maxhealth)
		-- Power		
		UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
		UnitEventHandlers.UNIT_AURA(self, unitid)
		UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
		UnitEventHandlers.UNIT_HEALTH(self, unitid)	
		UnitEventHandlers.UNIT_HEAL_PREDICTION(self, unitid)	
		-- Vehicle
		if UnitHasVehicleUI(unitid) then		-- Vehicle Support
			Graphics.Health:SetBackdropColor(0, 1, 0, 1)
			Graphics.Text:SetTextColor(0, .5, 0, 1)
		else
			Graphics.Text:SetTextColor(color.r*.5, color.g*.5, color.b*.5, 1)
			Graphics.Health:SetBackdropColor(color.r, color.g, color.b, 1)
		end
end

function SetHandlers(self)
	self:SetScript("OnShow", OnShowButton)
	self:SetScript("OnEvent", OnEvent)
	self:SetScript("OnUpdate", PollOnUpdate)
	for eventname, _ in pairs(UnitEventHandlers) do self:RegisterEvent(eventname) end
end

function OnShowButton(self)
	if not InCombatLockdown() then self:SetAttribute('*type1','target') end
	
	local unitid = self:GetAttribute("unit")
	if unitid then 
		self.UnitID = unitid
		UpdateButton(self, unitid)
	end
end

function OnEvent(self, event, ...)
	UnitEventHandlers[event](self, ...) 
end

function OnChangeAttribute(self, attribute, value)
	if attribute == "_wrapentered" then return	-- Clique filter
	elseif attribute == "unit" then 
		if not InCombatLockdown() then 
			self:SetAttribute('*type1','target')
			self:SetAttribute("toggleForVehicle", true)		-- Vehicle Support
		end
		
		ClickCastFrames[self] = true		-- Clique
		SetHandlers(self)
		
		local unitid = self:GetAttribute("unit")
		if unitid then 
			self.UnitID = unitid
			UpdateButton(self, unitid) 
		end
	end
end

------------------------------------
-- Unit Frame Artwork
------------------------------------
function CreateUnitFrameArtwork(self)
	local Graphics
	self:EnableMouse(true)
	self.info = {}
	
	-- Create Base Frames
	self.AuraWatcher = CreateFrame("Frame", nil, self)
	self.Graphics = CreateFrame("Frame", nil, self)
	self.auraUpdateTime = 0
	self.rangeUpdateTime = 0
	
	Graphics = self.Graphics
	Graphics:SetPoint("CENTER", self)
	Graphics:SetWidth(64)
	Graphics:SetHeight(64)
		
	-- Health Bar 
	Graphics.Health = CreateFrame("StatusBar", nil, Graphics)
	Graphics.Health:SetStatusBarTexture(artpath.."HealthStatusbar")
	Graphics.Health:SetBackdrop({bgFile = artpath.."HealthStatusbar"});
	Graphics.Health:SetOrientation("VERTICAL")
	Graphics.Health:SetPoint("CENTER", 0, -1)
	Graphics.Health:SetFrameLevel(Graphics:GetFrameLevel()-1)
	Graphics.Health:SetWidth(36)
	Graphics.Health:SetHeight(34)
	
	-- Heal Prediction Bar 
	Graphics.IncomingHeal = CreateFrame("StatusBar", nil, Graphics.Health)
	Graphics.IncomingHeal:SetStatusBarTexture(artpath.."HealthStatusbar")
	Graphics.IncomingHeal:SetOrientation("VERTICAL")
	Graphics.IncomingHeal:SetAllPoints(Graphics.Health)
	
	-- Overlay Anchor (To make layering easier)
	Graphics.OverlayAnchor = CreateFrame("StatusBar", nil, Graphics.Health)
	Graphics.OverlayAnchor:SetAllPoints()
	Graphics.OverlayAnchor:SetFrameLevel(Graphics.IncomingHeal:GetFrameLevel()+1)
	
	-- Power (Mana) Bar
	Graphics.Power = CreateFrame("StatusBar", nil, Graphics.OverlayAnchor)
	Graphics.Power:SetStatusBarTexture(artpath.."ManaStatusbar")
	Graphics.Power:SetBackdrop({bgFile = artpath.."ManaStatusbar"});
	Graphics.Power:SetOrientation("VERTICAL")
	Graphics.Power:SetPoint("CENTER", 16, 7)
	Graphics.Power:SetHeight(28)	
	Graphics.Power:SetWidth(7)	
	Graphics.Power:SetBackdropColor(0, 0, 0, 1)
	Graphics.Power:SetStatusBarColor(4/255, 78/255, 141/255, 1)

	Graphics.Power.Border = Graphics.Power:CreateTexture(nil, "OVERLAY")
	Graphics.Power.Border:SetTexture(artpath.."ManaOverlay")
	Graphics.Power.Border:SetPoint("TOPLEFT", -1,1)	
	Graphics.Power.Border:SetHeight(32)	
	Graphics.Power.Border:SetWidth(16)	

	-- Create Name Text Field
	Graphics.Text = Graphics.OverlayAnchor:CreateFontString(nil, "OVERLAY")
	Graphics.Text:SetFont(artpath.."LiberationSans-Regular.ttf", 10)
	Graphics.Text:SetPoint("BOTTOM", Graphics.Health, "BOTTOM", 0, 1)
	Graphics.Text:SetWidth(50)
	Graphics.Text:SetHeight(13)
	Graphics.Text:SetShadowColor(0,0,0,1) 
	Graphics.Text:SetShadowOffset(1,-1) 
	
	-- Create Mouseover Highlight
	Graphics.Glow = Graphics.OverlayAnchor:CreateTexture(nil, "HIGHLIGHT")
	Graphics.Glow:SetTexture("Interface/TARGETINGFRAME/UI-StatusBar")
	Graphics.Glow:SetVertexColor(.5,.5,.5,.5)
	Graphics.Glow:SetBlendMode("ADD")
	Graphics.Glow:SetAllPoints(Graphics.Health)

	-- Overlay Artwork
	Graphics.Overlay = Graphics.OverlayAnchor:CreateTexture(nil, "ARTWORK")
	Graphics.Overlay:SetTexture(artpath.."HealthOverlay")
	Graphics.Overlay:SetAllPoints(Graphics)
	
	-- Border/Selection Box Artwork
	Graphics.Border = Graphics:CreateTexture(nil, "ARTWORK")
	Graphics.Border:SetTexture(artpath.."Select")
	Graphics.Border:SetAllPoints()
	Graphics.Border:Hide()

	Graphics.Indicators = {}
	local Indicators = Graphics.Indicators
	
	-- Indicators Table
	Graphics.IndicatorLayer = CreateFrame("Frame", nil, Graphics)
	Graphics.IndicatorLayer:SetAllPoints(Graphics)
	Graphics.IndicatorLayer:Show()
	
	local Indicator, IndicatorIndex
	for index = 1, 6 do
		Indicators[index] = CreateFrame("Frame", nil, Graphics.IndicatorLayer)
		Indicator = Indicators[index]
		Indicator:SetWidth(16)
		Indicator:SetHeight(16)
		Indicator.Image = Indicator:CreateTexture(nil, "ARTWORK")
		Indicator.Image:SetAllPoints()
		Indicator.Text = Indicator:CreateFontString(nil, "OVERLAY")
		Indicator.Text:SetFont(artpath.."LiberationSans-Regular.ttf", 7, "OUTLINE")
		Indicator.Text:SetPoint("TOP", Indicator, "BOTTOM", 1, 7)
		Indicator.Text:SetWidth(12)
		Indicator.Text:SetHeight(12)
		Indicator:Hide()	
	end
	
	for index = 1, 3 do Indicators[index].Image:SetTexture(artpath.."DebuffDot") end
	for index = 2, 3 do Indicators[index]:SetPoint("RIGHT", Indicators[index-1], "CENTER", -1, 0) end
	Indicators[1]:SetPoint("TOP", 6, -7)
	
	for index = 4, 6 do Indicators[index].Image:SetTexture(artpath.."DebuffArrow") end
	for index = 5, 6 do Indicators[index]:SetPoint("RIGHT", Indicators[index-1], "CENTER", -3, 0) end
	Indicators[4]:SetPoint("BOTTOM", 11, 4)
	
	-- Aggro/Outline Artwork
	Graphics.Aggro = Graphics:CreateTexture(nil, "OVERLAY")
	Graphics.Aggro:SetTexture(artpath.."Aggro")
	Graphics.Aggro:SetVertexColor(1,0,0,1)
	Graphics.Aggro:SetAllPoints()
	Graphics.Aggro:Hide()
	
	-- Scripts
	self:SetScript("OnAttributeChanged", OnChangeAttribute)
end

-- Calls the Unit Frame Artwork function; Called by the secure config function
function CallArtworkFunction(self, frame, ...)
	CreateUnitFrameArtwork(_G[frame])
end

-- Initialization Function: Set Size and other properties of the Unit Button
-- Variables from addons are not accessible when this function is called.
SecureInitFunction = [[
	local header = self:GetParent()
	header:CallMethod('CallArtworkFunction', self:GetName())
	self:SetHeight(45)
	self:SetWidth(45)
]]

-- Creates the Group Header, which dynamically controls the creation of Unit Frames
function CreateGroupHeader()
	local RaidGroupHeader = CreateFrame("Frame", addonname.."_GroupHeader", UIParent, "SecureGroupHeaderTemplate")
	local HandleFrame = CreateFrame("Frame", addonname.."_Handle", RaidGroupHeader)
	
	RaidGroupHeader:SetPoint("CENTER", UIParent)
	RaidGroupHeader:SetMovable(true)
	RaidGroupHeader:SetClampedToScreen(true)

	-- RaidGroupHeader Frame Header
	RaidGroupHeader:SetAttribute("template", "SecureUnitButtonTemplate")
	RaidGroupHeader:SetAttribute("templateType", "Button")
	RaidGroupHeader:SetAttribute("groupFilter", "1,2,3,4,5,6,7,8")
	RaidGroupHeader:SetAttribute("showRaid", sgConfig.ShowRaid) -- When true, the group header is shown when the player is in a raid.
	RaidGroupHeader:SetAttribute("showParty", sgConfig.ShowParty) -- When true, the group header is shown when the player is in a party. This attribute doesnt imply showRaid but can work alongside it.
	RaidGroupHeader:SetAttribute("showSolo", sgConfig.ShowSolo) -- When true, the header is shown when the player is not in any group. This option implies showPlayer.
	RaidGroupHeader:SetAttribute("showPlayer", sgConfig.ShowPlayer) -- When true, the header includes the player when not in a raid (normally, the player would not be visible in a party listing).
	RaidGroupHeader:SetAttribute("point", sgConfig.Point)
	RaidGroupHeader:SetAttribute("unitsPerColumn",5)
	RaidGroupHeader:SetAttribute("maxColumns",8)
	RaidGroupHeader:SetAttribute("columnAnchorPoint","TOP") -- The anchor point for each new column. A value of LEFT will cause the columns to grow to the right.
	RaidGroupHeader.CallArtworkFunction = CallArtworkFunction	-- This function is called from the init function
	RaidGroupHeader:SetAttribute("initialConfigFunction", SecureInitFunction) 		-- Init function (Secure)
			
	-- Handle Frame
	HandleFrame:SetPoint("TOP", RaidGroupHeader, "TOP", 0, 0)
	HandleFrame:SetPoint("RIGHT", RaidGroupHeader, "RIGHT", 18, 0)
	HandleFrame:SetWidth(16)
	HandleFrame:SetHeight(16)

	HandleFrame:EnableMouse(true)	-- /run ShotGlassRaidFrames.HandleFrame:EnableMouse(false) -- to lock the frame, in game
	HandleFrame:SetScript("OnMouseDown",function(self, button) RaidGroupHeader:StartMoving() end)
	HandleFrame:SetScript("OnMouseUp",function(self, button) RaidGroupHeader:StopMovingOrSizing() end)
	
	HandleFrame.Texture = HandleFrame:CreateTexture(nil, "ARTWORK")
	HandleFrame.Texture:SetTexture(artpath.."CornerHandle")
	HandleFrame.Texture:SetAllPoints(HandleFrame)
	RaidGroupHeader.HandleFrame = HandleFrame
	
	RaidGroupHeader:Show()
	return RaidGroupHeader
end

function sgShowHandle(show)
	if show == false then
		ShotGlassRaidFrames.HandleFrame:Hide()
	else
		ShotGlassRaidFrames.HandleFrame:Show()
	end
    sgConfig.ShowDrag = show
end -- sgShowHandle

function sgParseCommand(cmd)
	local param1, param2, param3 = cmd:upper():match("^([%S]+)%s*([%S]*)%s*(.*)$");
	param1, param2, param3 = param1 or "", param2 or "", param3 or "";

	if param1 == "TOGGLE" or param1 == "" then
		sgShowHandle(not sgConfig.ShowDrag)
	elseif param1 == "SETPOINT" then
		if param2 == "" then param2 = 0 else param2 = tonumber(param2) end
		if param3 == "" then param3 = -260 else param3 = tonumber(param3) end -- personal preference
		ShotGlassRaidFrames:ClearAllPoints()
		ShotGlassRaidFrames:SetPoint("CENTER", param2, param3)
	elseif param1 == "SCALE" then
		if param2 == "" then sgConfig.Defaultscale = 1 else sgConfig.Defaultscale = tonumber(param2) end
	elseif param1 == "ORDER" then
		if param2 == "RIGHT" or param2 == "LEFT" then
			sgConfig.Point = param2
			ShotGlassRaidFrames:SetAttribute("point", sgConfig.Point)
		end
	elseif param1 == "HIDE" then
		ShotGlassRaidFrames:Hide()
	elseif param1 == "SHOW" then
		if param2 == "SOLO" then
			sgConfig.ShowSolo = not sgConfig.ShowSolo
			ShotGlassRaidFrames:SetAttribute("showSolo", sgConfig.ShowSolo)
			print("ShotGlass display for solo: ", sgConfig.ShowSolo)
		elseif param2 == "PARTY" then
			sgConfig.ShowParty = not sgConfig.ShowParty
			ShotGlassRaidFrames:SetAttribute("showParty", sgConfig.ShowParty)
			print("ShotGlass display for party: ", sgConfig.ShowParty)
		elseif param2 == "RAID" then
			sgConfig.ShowRaid = not sgConfig.ShowRaid
			ShotGlassRaidFrames:SetAttribute("showRaid", sgConfig.ShowRaid)
			print("ShotGlass display for raid: ", sgConfig.ShowRaid)
		elseif param2 == "PLAYER" then
			sgConfig.ShowPlayer = not sgConfig.ShowPlayer
			ShotGlassRaidFrames:SetAttribute("showPlayer", sgConfig.ShowPlayer)
			print("ShotGlass display of player: ", sgConfig.ShowPlayer)
		else
			ShotGlassRaidFrames:Show()
		end
	end -- if
end -- function sgParseCommand

ShotGlassRaidFrames = CreateGroupHeader()
ShotGlassRaidFrames.TrackedAuras = TrackedAuras

SLASH_SHOTGLASS1 = "/shotglass";
SlashCmdList["SHOTGLASS"] = sgParseCommand;

sgLoadFrame = CreateFrame("frame") 
sgLoadFrame:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, event, ...) end end) 
sgLoadFrame:RegisterEvent("ADDON_LOADED")
function sgLoadFrame:ADDON_LOADED(event, addon)
	if (addon ~= "ShotGlass") then return end
	if sgConfiguration == nil then
		sgConfiguration = sgConfig -- load defaults
	else
		sgConfig = sgConfiguration
		sgShowHandle(sgConfig.ShowDrag)
		ShotGlassRaidFrames:SetAttribute("point", sgConfig.Point)
		ShotGlassRaidFrames:SetAttribute("showSolo", sgConfig.ShowSolo)
		ShotGlassRaidFrames:SetAttribute("showParty", sgConfig.ShowParty)
		ShotGlassRaidFrames:SetAttribute("showRaid", sgConfig.ShowRaid)
		ShotGlassRaidFrames:SetAttribute("showPlayer", sgConfig.ShowPlayer)
	end
	self:RegisterEvent("PLAYER_LOGOUT");
	-- Cleanup
	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
end -- ADDON_LOADED

function sgLoadFrame:PLAYER_LOGOUT()
	sgConfiguration = sgConfig
end  -- PLAYER_LOGOUT
